@@ -28,6 +28,15 @@ import re
 
 DOCTYPE = '<?xml version="1.0" encoding="UTF-8"?>'
 XMLNS = 'http://docs.openstack.org/identity/api/v2.0'
+XMLNS_LIST = [
+    {
+        'value': 'http://docs.openstack.org/identity/api/v2.0'
+    },
+    {
+        'prefix': 'OS-KSADM',
+        'value': 'http://docs.openstack.org/identity/api/ext/OS-KSADM/v1.0',
+    },
+]
 
 PARSER = etree.XMLParser(
     resolve_entities=False,
@@ -62,22 +71,40 @@ class XmlDeserializer(object):
     def __call__(self, xml_str):
         """Returns a dictionary populated by decoding the given xml string."""
         dom = etree.fromstring(xml_str.strip(), PARSER)
-        return self.walk_element(dom)
+        return self.walk_element(dom, True)
+
+    def _deserialize_links(self, links):
+        return dict((x.attrib['rel'], x.attrib['href']) for x in links)
 
     @staticmethod
-    def _tag_name(tag):
-        """Remove the namespace from the tagname.
+    def _tag_name(tag, namespace):
+        """Returns a tag name.
 
-        TODO(dolph): We might care about the namespace at some point.
-
-        >>> XmlDeserializer._tag_name('{xmlNamespace}tagName')
-        'tagName'
+        The tag name may contain the namespace prefix or not, which can
+        be determined by specifying the parameter namespace.
 
         """
         m = re.search('[^}]+$', tag)
-        return m.string[m.start():]
+        tag_name = m.string[m.start():]
+        if not namespace:
+            return tag_name
+        bracket = re.search('[^{]+$', tag)
+        ns = m.string[bracket.start():m.start() - 1]
+        #If the namespace is
+        #http://docs.openstack.org/identity/api/ext/OS-KSADM/v1.0
+        #for the root element, a prefix needs to add in front of the tag name.
+        prefix = None
+        for xmlns in XMLNS_LIST:
+            if xmlns['value'] == ns:
+                prefix = xmlns.get('prefix', None)
+                break
+        if prefix is not None:
+            return '%(PREFIX)s:%(tag_name)s' \
+                % {'PREFIX': prefix, 'tag_name': tag_name}
+        else:
+            return tag_name
 
-    def walk_element(self, element):
+    def walk_element(self, element, namespace=False):
         """Populates a dictionary by walking an etree element."""
         values = {}
         for k, v in element.attrib.iteritems():
@@ -96,12 +123,44 @@ class XmlDeserializer(object):
 
         # current spec does not have attributes on an element with text
         values = values or text or {}
+        decoded_tag = XmlDeserializer._tag_name(element.tag, namespace)
+        list_item_tag = None
+        if decoded_tag[-1] == 's' and len(values) == 0:
+            # FIXME(gyee): special-case lists for now unti we
+            # figure out how to properly handle them.
+            # If any key ends with an 's', we are assuming it is a list.
+            # List element have no attributes.
+            values = list(values)
+            if decoded_tag == 'policies':
+                list_item_tag = 'policy'
+            else:
+                list_item_tag = decoded_tag[:-1]
 
+        if decoded_tag == 'links':
+            return {'links': self._deserialize_links(element)}
+
+        links = None
         for child in [self.walk_element(x) for x in element
                       if not isinstance(x, ENTITY_TYPE)]:
-            values = dict(values.items() + child.items())
+            if list_item_tag:
+                # FIXME(gyee): special-case lists for now until we
+                # figure out how to properly handle them.
+                # If any key ends with an 's', we are assuming it is a list.
+                if list_item_tag in child:
+                    values.append(child[list_item_tag])
+                else:
+                    links = child['links']
+            else:
+                values = dict(values.items() + child.items())
 
-        return {XmlDeserializer._tag_name(element.tag): values}
+        d = {XmlDeserializer._tag_name(element.tag, namespace): values}
+
+        if links:
+            d['links'] = links
+            d['links'].setdefault('next')
+            d['links'].setdefault('previous')
+
+        return d
 
 
 class XmlSerializer(object):
@@ -111,25 +170,52 @@ class XmlSerializer(object):
         Optionally, namespace the etree by specifying an ``xmlns``.
 
         """
+        links = None
         # FIXME(dolph): skipping links for now
         for key in d.keys():
             if '_links' in key:
                 d.pop(key)
+            # FIXME(gyee): special-case links in collections
+            if 'links' == key:
+                if links:
+                    # we have multiple links
+                    raise Exception('Multiple links found')
+                links = d.pop(key)
 
         assert len(d.keys()) == 1, ('Cannot encode more than one root '
                                     'element: %s' % d.keys())
 
         # name the root dom element
         name = d.keys()[0]
-
+        m = re.search('[^:]+$', name)
+        root_name = m.string[m.start():]
+        prefix = m.string[0:m.start() - 1]
+        for ns in XMLNS_LIST:
+            if prefix == ns.get('prefix', None):
+                xmlns = ns['value']
+                break
         # only the root dom element gets an xlmns
-        root = etree.Element(name, xmlns=(xmlns or XMLNS))
+        root = etree.Element(root_name, xmlns=(xmlns or XMLNS))
 
         self.populate_element(root, d[name])
 
+        # FIXME(gyee): special-case links for now
+        if links:
+            self._populate_links(root, links)
+
         # TODO(dolph): you can get a doctype from lxml, using ElementTrees
         return '%s\n%s' % (DOCTYPE, etree.tostring(root, pretty_print=True))
 
+    def _populate_links(self, element, links_json):
+        links = etree.Element('links')
+        for k, v in links_json.iteritems():
+            if v:
+                link = etree.Element('link')
+                link.set('rel', unicode(k))
+                link.set('href', unicode(v))
+                links.append(link)
+        element.append(links)
+
     def _populate_list(self, element, k, v):
         """Populates an element with a key & list value."""
         # spec has a lot of inconsistency here!
@@ -143,18 +229,28 @@ class XmlSerializer(object):
                 container = etree.Element(k)
                 element.append(container)
             name = k[:-1]
-        elif k == 'serviceCatalog':
+        elif k == 'serviceCatalog' or k == 'catalog':
             # xsd compliance: <serviceCatalog> contains <service>s
             container = etree.Element(k)
             element.append(container)
             name = 'service'
+        elif k == 'roles' and element.tag == 'user':
+            name = 'role'
+        elif k == 'endpoints' and element.tag == 'service':
+            name = 'endpoint'
         elif k == 'values' and element.tag[-1] == 's':
             # OS convention is to contain lists in a 'values' element,
             # so the list itself can have attributes, which is
             # unnecessary in XML
             name = element.tag[:-1]
         elif k[-1] == 's':
-            name = k[:-1]
+            container = etree.Element(k)
+            element.append(container)
+            if k == 'policies':
+                # need to special-case policies since policie is not a word
+                name = 'policy'
+            else:
+                name = k[:-1]
         else:
             name = k
 
@@ -165,9 +261,13 @@ class XmlSerializer(object):
 
     def _populate_dict(self, element, k, v):
         """Populates an element with a key & dictionary value."""
-        child = etree.Element(k)
-        self.populate_element(child, v)
-        element.append(child)
+        if k == 'links':
+            # links is a special dict
+            self._populate_links(element, v)
+        else:
+            child = etree.Element(k)
+            self.populate_element(child, v)
+            element.append(child)
 
     def _populate_bool(self, element, k, v):
         """Populates an element with a key & boolean value."""
@@ -196,6 +296,8 @@ class XmlSerializer(object):
             self._populate_sequence(element, value)
         elif isinstance(value, dict):
             self._populate_tree(element, value)
+        elif isinstance(value, basestring):
+            element.text = unicode(value)
 
     def _populate_sequence(self, element, l):
         """Populates an etree with a sequence of elements, given a list."""
@@ -203,6 +305,8 @@ class XmlSerializer(object):
         name = element.tag
         if element.tag[-1] == 's':
             name = element.tag[:-1]
+            if name == 'policie':
+                name = 'policy'
 
         for item in l:
             child = etree.Element(name)
