@@ -15,22 +15,20 @@
 # under the License.
 
 
-from migrate import *
 from sqlalchemy import *
 
 
-from keystone.common import sql
-
-
 def upgrade(migrate_engine):
     # Upgrade operations go here. Don't create your own engine; bind
 
     meta = MetaData()
     meta.bind = migrate_engine
-    dialect = migrate_engine.url.get_dialect().name
     token = Table('token', meta, autoload=True)
-    valid = Column("valid", Boolean(), ColumnDefault(True), nullable=False)
-    token.create_column(valid)
+    # creating the column immediately with nullable=False fails with
+    # PostgreSQL (LP 1068181), so do it in two steps instead
+    valid = Column("valid", Boolean(), ColumnDefault(True), nullable=True)
+    valid.create(token, populate_default=True)
+    valid.alter(type=Boolean(), default=True, nullable=False)
 
 
 def downgrade(migrate_engine):
