@@ -1,4 +1,6 @@
-# Copyright 2012 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2012 OpenStack LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -12,12 +14,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import functools
+
 from keystone.common import sql
+from keystone.common.sql import migration
 from keystone import exception
 from keystone.policy.backends import rules
 
 
-class PolicyModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
+class PolicyModel(sql.ModelBase, sql.DictBase):
     __tablename__ = 'policy'
     attributes = ['id', 'blob', 'type']
     id = sql.Column(sql.String(64), primary_key=True)
@@ -26,35 +31,45 @@ class PolicyModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
     extra = sql.Column(sql.JsonBlob())
 
 
-class Policy(rules.Policy):
+class Policy(sql.Base, rules.Policy):
+    # Internal interface to manage the database
+    def db_sync(self):
+        migration.db_sync()
 
-    @sql.handle_conflicts(conflict_type='policy')
+    @sql.handle_conflicts(type='policy')
     def create_policy(self, policy_id, policy):
-        with sql.session_for_write() as session:
+        session = self.get_session()
+
+        with session.begin():
             ref = PolicyModel.from_dict(policy)
             session.add(ref)
+            session.flush()
 
-            return ref.to_dict()
+        return ref.to_dict()
 
     def list_policies(self):
-        with sql.session_for_read() as session:
-            refs = session.query(PolicyModel).all()
-            return [ref.to_dict() for ref in refs]
+        session = self.get_session()
+
+        refs = session.query(PolicyModel).all()
+        return [ref.to_dict() for ref in refs]
 
     def _get_policy(self, session, policy_id):
         """Private method to get a policy model object (NOT a dictionary)."""
-        ref = session.get(PolicyModel, policy_id)
-        if not ref:
+        try:
+            return session.query(PolicyModel).filter_by(id=policy_id).one()
+        except sql.NotFound:
             raise exception.PolicyNotFound(policy_id=policy_id)
-        return ref
 
     def get_policy(self, policy_id):
-        with sql.session_for_read() as session:
-            return self._get_policy(session, policy_id).to_dict()
+        session = self.get_session()
 
-    @sql.handle_conflicts(conflict_type='policy')
+        return self._get_policy(session, policy_id).to_dict()
+
+    @sql.handle_conflicts(type='policy')
     def update_policy(self, policy_id, policy):
-        with sql.session_for_write() as session:
+        session = self.get_session()
+
+        with session.begin():
             ref = self._get_policy(session, policy_id)
             old_dict = ref.to_dict()
             old_dict.update(policy)
@@ -62,10 +77,14 @@ class Policy(rules.Policy):
             ref.blob = new_policy.blob
             ref.type = new_policy.type
             ref.extra = new_policy.extra
+            session.flush()
 
         return ref.to_dict()
 
     def delete_policy(self, policy_id):
-        with sql.session_for_write() as session:
+        session = self.get_session()
+
+        with session.begin():
             ref = self._get_policy(session, policy_id)
             session.delete(ref)
+            session.flush()
